/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include <media_content.h>
#include <app_debug.h>

#include "data/datamgr.h"

enum _filter_type {
	E_FILTER_FOLDER = 0,
	E_FILTER_MEDIA
};

struct folderdata {
	Eina_List *folder_list;

	char *parent_id;

	const char *media_type;
	int source_type;
};

static bool _create_filter(struct folderdata *fd, filter_h *filter,
			const char *cond, int filter_type)
{
	filter_h tmp_filter;
	int ret;
	char buf[1024];

	ret = media_filter_create(&tmp_filter);
	if (ret != MEDIA_CONTENT_ERROR_NONE) {
		_ERR("failed to create media filter");
		return false;
	}

	snprintf(buf, sizeof(buf), "%s", fd->media_type);

	if (fd->source_type != E_SOURCE_ALL) {
		char s1[256];

		snprintf(s1, sizeof(s1), " AND MEDIA_STORAGE_TYPE=%d",
					fd->source_type);

		strcat(buf, s1);
	}

	if (cond) {
		char s2[256];

		snprintf(s2, sizeof(s2), " AND %s", cond);

		strcat(buf, s2);
	}

	media_filter_set_condition(tmp_filter, buf,
			MEDIA_CONTENT_COLLATE_DEFAULT);

	if (filter_type == E_FILTER_FOLDER) {
		media_filter_set_order(tmp_filter, MEDIA_CONTENT_ORDER_ASC,
				FOLDER_NAME, MEDIA_CONTENT_COLLATE_DEFAULT);
	}

	*filter = tmp_filter;

	return true;
}

static void _destroy_folder_list(struct folderdata *fd)
{
	struct folder_info *fi;

	EINA_LIST_FREE(fd->folder_list, fi) {
		free(fi->id);
		free(fi->name);
		free(fi->path);
		free(fi->parent_id);

		free(fi);
	}

	fd->folder_list = NULL;
}

static bool _get_folder_id(media_folder_h folder, void *data)
{
	struct folderdata *fd;

	if (!data)
		return false;

	fd = data;

	if (media_folder_get_folder_id(folder, &(fd->parent_id))
		!= MEDIA_CONTENT_ERROR_NONE) {
		_ERR("failed to fetch folder id");
		return false;
	}

	return true;
}

static bool _get_each_folder_info(media_folder_h folder, void *data)
{
	filter_h filter;
	struct folderdata *fd;
	struct folder_info *fi;

	if (!data)
		return false;

	fd = data;

	fi = calloc(1, sizeof(*fi));
	if (!fi) {
		_ERR("failed to allocate memory");
		return false;
	}

	if (media_folder_get_folder_id(folder, &(fi->id))
		!= MEDIA_CONTENT_ERROR_NONE) {
		_ERR("failed to fetch folder id");
		free(fi);
		return false;
	}

	if (media_folder_get_name(folder, &(fi->name))
		!= MEDIA_CONTENT_ERROR_NONE) {
		_ERR("failed to fetch folder name");
		free(fi);
		return false;
	}

	if (media_folder_get_path(folder, &(fi->path))
		!= MEDIA_CONTENT_ERROR_NONE) {
		_ERR("failed to fetch folder path");
		free(fi);
		return false;
	}

	if (media_folder_get_parent_folder_id(folder, &(fi->parent_id))
		!= MEDIA_CONTENT_ERROR_NONE) {
		_ERR("failed to fetch parent folder id");
		free(fi);
		return false;
	}

	if (!_create_filter(fd, &filter, NULL, E_FILTER_MEDIA)) {
		_ERR("failed to create filter");
		free(fi);
		return false;
	}

	if (media_folder_get_media_count_from_db(fi->id, filter, &(fi->count))
		!= MEDIA_CONTENT_ERROR_NONE) {
		_ERR("failed to get media count");
		free(fi);
		media_filter_destroy(filter);
		return false;
	}

	fd->folder_list = eina_list_append(fd->folder_list, fi);

	media_filter_destroy(filter);

	return true;
}

static bool _find_folder_id(struct folderdata *fd, const char *path)
{
	filter_h filter;
	int ret;
	char buf[1024];

	snprintf(buf, sizeof(buf), "FOLDER_PATH=\"%s\"", path);

	if (!_create_filter(fd, &filter, buf, E_FILTER_FOLDER)) {
		_ERR("failed to create filter");
		return false;
	}

	ret = media_folder_foreach_folder_from_db(filter,
				_get_folder_id, fd);
	if (ret != MEDIA_CONTENT_ERROR_NONE) {
		_ERR("failed to get folder info");
		media_filter_destroy(filter);
		return false;
	}

	media_filter_destroy(filter);

	return true;
}

static bool _get_sub_folder_list(struct folderdata *fd)
{
	filter_h filter;
	int ret;
	char buf[1024];

	snprintf(buf, sizeof(buf), "FOLDER_PARENT_FOLDER_ID=\"%s\"",
				fd->parent_id);

	if (!_create_filter(fd, &filter, buf, E_FILTER_FOLDER)) {
		_ERR("failed to create filter");
		return false;
	}

	ret = media_folder_foreach_folder_from_db(filter,
				_get_each_folder_info, fd);
	if (ret != MEDIA_CONTENT_ERROR_NONE) {
		_ERR("failed to get folder info");
		_destroy_folder_list(fd);
		media_filter_destroy(filter);
		return false;
	}

	media_filter_destroy(filter);

	return true;
}

static void *_create(const char *media_type, int source_type)
{
	struct folderdata *fd;

	if (!media_type || source_type < 0 || source_type >= E_SOURCE_MAX) {
		_ERR("invalid argument");
		return NULL;
	}

	fd = calloc(1, sizeof(*fd));
	if (!fd) {
		_ERR("failed to allocate fd");
		return NULL;
	}

	fd->media_type = media_type;
	fd->source_type = source_type;

	return (void *)fd;
}

static void _destroy(void *handle)
{
	struct folderdata *fd;

	if (!handle) {
		_ERR("failed to get folderdata handle");
		return;
	}

	fd = handle;

	_destroy_folder_list(fd);

	free(fd->parent_id);
	free(fd);
}

static Eina_List *_get_list(void *handle)
{
	struct folderdata *fd;

	if (!handle) {
		_ERR("failed to get folderdata handle");
		return NULL;
	}

	fd = handle;

	return fd->folder_list;
}

static int _get_count(void *handle, int type)
{
	struct folderdata *fd;

	if (!handle) {
		_ERR("failed to get folderdata handle");
		return -1;
	}

	fd = handle;

	return eina_list_count(fd->folder_list);
}

static void _free_group(Eina_List *list)
{
	eina_list_free(list);
}

static Eina_List *_get_group(void *handle, int type, void *data)
{
	struct folderdata *fd;
	int ret;

	if (!handle) {
		_ERR("failed to get folderdata handle");
		return NULL;
	}

	if (type < 0 || type > E_FOLDER_MAX || !data) {
		_ERR("invalid argument");
		return NULL;
	}

	fd = handle;

	if (fd->folder_list) {
		_destroy_folder_list(fd);
		free(fd->parent_id);
	}

	ret = media_content_connect();
	if (ret != MEDIA_CONTENT_ERROR_NONE) {
		_ERR("failed to connect to media content");
		return false;
	}

	if (!_find_folder_id(fd, data)) {
		_ERR("failed to find folder id");
		media_content_disconnect();
		return false;
	}

	if (!_get_sub_folder_list(fd)) {
		_ERR("failed to get child folder list");
		media_content_disconnect();
		return false;
	}

	media_content_disconnect();

	return eina_list_clone(fd->folder_list);
}

static struct data_ops _ops = {
	.create = _create,
	.destroy = _destroy,
	.get_list = _get_list,
	.get_count = _get_count,
	.get_group = _get_group,
	.free_group = _free_group,
};

struct data_ops *folderdata_get_ops(void)
{
	return &_ops;
};
